#include <iostream>
#include <vector>
#include <algorithm>

using namespace std;

struct Segment {
    int l, r;
    explicit Segment() {

    }
    Segment(int a, int b) {
        l = a; r = b;
    }
};

struct Info {
    Segment minRight, maxLeft;
    Info(int a, int b) {
        minRight = {a, a};
        maxLeft = {b, b};
    }
    Info(Segment a, Segment b) {
        minRight = a;
        maxLeft = b;
    }
};

const int INF = 1e9;

bool canMerge(Segment a, Segment b) {
    if (a.r <= b.l || b.r <= a.l) {
        return true;
    }
    return false;
}

Segment merge(Segment a, Segment b) {
    return Segment(min(a.l, b.l), max(a.r, b.r));
}

void relaxLeft(Segment &a, Segment b) {
    if (b.l > a.l) {
        a = b;
    } else if (b.l == a.l) {
        if (b.r < a.r) {
            a = b;
        }
    }
}

void relaxRight(Segment &a, Segment b) {
    if (b.r < a.r) {
        a = b;
    } else if (b.r == a.r) {
        if (b.l > a.l) {
            a = b;
        }
    }
}

Info dfs(vector<vector<int>> &upgrades, vector<vector<int>> &childrens, int v) {
    if (childrens[v].size() == 0) {
        return Info(*upgrades[v].begin(), upgrades[v].back());
    }
    if (childrens[v].size() == 1) {
        return dfs(upgrades, childrens, childrens[v][0]);
    }
    Info left = dfs(upgrades, childrens, childrens[v][0]);
    Info right = dfs(upgrades, childrens, childrens[v][1]);
    Segment maxLeft = Segment(-INF, INF);
    Segment minRight = Segment(-INF, INF);
    bool flag = false;
    if (canMerge(left.minRight, right.minRight)) {
        flag = true;
        Segment x = merge(left.minRight, right.minRight);
        relaxLeft(maxLeft, x);
        relaxRight(minRight, x);
    }
    if (canMerge(left.minRight, right.maxLeft)) {
        flag = true;
        Segment x = merge(left.minRight, right.maxLeft);
        relaxLeft(maxLeft, x);
        relaxRight(minRight, x);
    }
    if (canMerge(left.maxLeft, right.minRight)) {
        flag = true;
        Segment x = merge(left.maxLeft, right.minRight);
        relaxLeft(maxLeft, x);
        relaxRight(minRight, x);
    }
    if (canMerge(left.maxLeft, right.maxLeft)) {
        flag = true;
        Segment x = merge(left.maxLeft, right.maxLeft);
        relaxLeft(maxLeft, x);
        relaxRight(minRight, x);
    }
    if (!flag) {
        cout << "No" << endl;
        exit(0);
    }
    return Info(minRight, maxLeft);
}

int main() {
    int n; cin >> n;
    vector<vector<int>> upgrades(n);
    vector<vector<int>> childrens(n);
    for (int i = 0; i < n; i++) {
        int t; cin >> t;
        if (t == 1) {
            int k; cin >> k;
            for (int j = 0; j < k; j++) {
                int x; cin >> x; x--;
                childrens[i].push_back(x);
            }
        }
        if (t == 2) {
            int m; cin >> m;
            if (m == 0) {
                cout << "No" << endl;
                return 0;
            }
            for (int j = 0; j < m; j++) {
                int x; cin >> x;
                upgrades[i].push_back(x);
            }
            sort(upgrades[i].begin(), upgrades[i].end());
        }
    }
    Info i = dfs(upgrades, childrens, 0);
    cout << "Yes" << endl;
    // cout << i.minRight.l << ' ' << i.minRight.r << endl;
    // cout << i.maxLeft.l << ' ' << i.maxLeft.r << endl;
}
