#pragma GCC optimize("O3")
#include <iostream>
#include <vector>
#include <queue>
#include <ios>
#include <set>
#include <cassert>
#include <algorithm>
#include <random>
using namespace std;

int type;
vector<vector<char>> field;
bool game_over = false;
int game_number = 0;
int n, m;

void make_move(int x, int y) {
	assert(!field[x][y]);

	for (int i = x; i < n; i++) {
		for (int j = y; j < n; j++) {
			field[i][j] = true;
		}
	}

	if (x == 0 && y == 0)
		game_over = true;
}

int get_amount_painted(int x, int y) {
	assert(!field[x][y]);
	int amount = 0;

	for (int i = x; i < n; i++) {
		for (int j = y; j < n; j++) {
			if (!field[i][j]) {
				amount++;
			}
		}
	}
	return amount;
}

vector<pair<int, int>> get_good_moves() {
	vector<pair<int, pair<int, int>>> all_moves;
	for (int i = n - 1; i >= 0; i--) {
		for (int j = n - 1; j >= 0; j--) {
			if (i == 0 && j == 0)
				continue;
			if (field[i][j] == true)
				continue;
			int amount = get_amount_painted(i, j);
			all_moves.push_back({ amount, {i, j} });
		}
	}
	stable_sort(all_moves.begin(), all_moves.end());
	vector<pair<int, int>> good_moves;
	for (int i = 0; i < all_moves.size(); i++) {
		auto [amount, move] = all_moves[i];
		if (amount > 5 && i >= 16)
			break;
		good_moves.push_back(move);
	}
	random_shuffle(good_moves.begin(), good_moves.end());
	return good_moves;
}

string message;
int message_pointer = 0;


pair<vector<pair<int, int>>, int> get_good_moves_and_max_encoded() {
	auto good_moves = get_good_moves();
	int max_encoded = 0;
	while (max_encoded + 1 <= m - message_pointer &&
			(1 << (max_encoded + 1)) <= good_moves.size()) {
		max_encoded++;
	}
	return { good_moves, max_encoded };
}

void make_move_and_encode_bits() {
	assert(type == 1);
	auto [good_moves, max_encoded] = get_good_moves_and_max_encoded();
	if (max_encoded == 0) {
		make_move(0, 0);
		cout << "1 1" << endl;
		return;
	}
	int bits = 0;
	for (int i = 0; i < max_encoded; i++, message_pointer++) {
		bits *= 2;
		bits += (message[message_pointer] - '0');
	}
	assert(bits < good_moves.size());
	auto [x, y] = good_moves[bits];
	make_move(x, y);
	cout << x + 1 << " " << y + 1 << endl;
}

void decode_bits() {
	int x, y;
	cin >> x >> y;
	x--; y--;
	auto [good_moves, max_encoded] = get_good_moves_and_max_encoded();
	if (max_encoded != 0) {
		int index = find(good_moves.begin(), good_moves.end(), make_pair(x, y)) -
			good_moves.begin();
		assert(index != good_moves.size());
		vector<bool> bits;
		for (int i = 0; i < max_encoded; i++) {
			bits.push_back(index % 2);
			index /= 2;
		}
		reverse(bits.begin(), bits.end());
		for (bool bit : bits) {
			message += (char)('0' + bit);
			message_pointer = message.size();
		}
	}
	make_move(x, y);
}

void execute_bot_move() {
	int x, y;
	cin >> x >> y;
	x--; y--;
	make_move(x, y);
}

void reset_game() {
	game_over = false;
	game_number++;
	for (int i = 0; i < n; i++)
		fill(field[i].begin(), field[i].end(), false);
}

int main() {
	ios_base::sync_with_stdio(0); cin.tie(0); cout.tie(0);
	srand(0);
	cin >> type;
	int k;
	cin >> n >> k >> m;
	field.resize(n, vector<char>(n));
	if (type == 1) {
		cin >> message;
		while (message_pointer != message.size()) {
			if (game_over) {
				reset_game();
				if (game_number % 2 == 1) {
					execute_bot_move();
					//assert(false);
				}
			}

			while (!game_over) {
				make_move_and_encode_bits();
				if (!game_over)
					execute_bot_move();
			}
		}
		cout << 0 << endl;
	}
	else {
		int games_amount;
		cin >> games_amount;
		while (true) {
			if (game_over) {
				reset_game();
				if (game_number == games_amount)
					break;
			}
			if (game_number % 2 == 1)
				execute_bot_move();
			while (!game_over) {
				decode_bits();
				if (!game_over)
					execute_bot_move();
			}
		}
		assert(message.size() == m);
		cout << message;
	}
}