#include <iostream>
#include <vector>
#include <algorithm>
#include <string>
#include <set>

using namespace std;
using ll = long long;

struct Node
{
	ll sum;
	ll add;
	Node(ll s = 0) { sum = s; add = 0; }

};

struct SegmentTree
{
	vector<Node> tree;

	Node merge(Node n1, Node n2)
	{
		return Node(n1.sum + n2.sum);
	}

	void push(ll v)
	{
		tree[2 * v + 1].sum += tree[v].add;
		tree[2 * v + 2].sum += tree[v].add;
		tree[2 * v + 1].add += tree[v].add;
		tree[2 * v + 2].add += tree[v].add;
		tree[v].add = 0;
	}

	void init(ll v, ll l, ll r, vector<ll>& start)
	{
		if (l == r)
		{
			tree[v] = Node(start[l]);
			return;
		}
		push(v);
		ll m = (r + l) / 2;

		init(2 * v + 1, l, m, start);
		init(2 * v + 2, m + 1, r, start);

		tree[v] = merge(tree[2 * v + 1], tree[2 * v + 2]);
	}

	Node query(ll v, ll l, ll r, ll id)
	{
		if (l == r && l == id)
		{
			return tree[v];
		}
		if (r < id || id < l)
			return Node(0);

		push(v);
		ll m = (r + l) / 2;

		return merge(query(2 * v + 1, l, m, id),
			query(2 * v + 2, m + 1, r, id));

	}

	void add(ll v, ll l, ll r, ll ql, ll qr, ll val)
	{
		if (ql <= l && r <= qr)
		{
			tree[v].sum += val;
			tree[v].add += val;
			return;
		}
		if (ql > r || l > qr)
			return;
		push(v);
		ll m = (r + l) / 2;

		add(2 * v + 1, l, m, ql, qr, val);
		add(2 * v + 2, m + 1, r, ql, qr, val);

		tree[v] = merge(tree[2 * v + 1], tree[2 * v + 2]);
	}
};

const ll inf = 1e18;

signed main()
{
	ll k, n; cin >> k >> n;
	vector<vector<ll>> a(k, vector<ll>(k, 0));
	for (int i = 0; i < k; ++i)
		for (int j = 0; j < k; ++j)
		{
			char c; cin >> c;
			a[i][j] = c - '0';
		}
	string s; cin >> s;
	vector<ll> l(n, -1);
	vector<ll> pos(k, -1);
	for (int i = 0; i < n; ++i)
	{
		for (int j = 0; j < k; ++j)
			if (a[j][s[i] - 'a'])
				l[i] = max(l[i], pos[j]);
		pos[s[i] - 'a'] = i;
	}
	string ans = s.substr(0, 1);
	ll t = 1;
	while (t < n)
	{
		pair<char, ll> p = { s[t],-t };
		for (int i = t; i < n; ++i)
		{//t i
			bool may = true;
			vector<pair<ll, ll>> q;
			for (int j = t; j <= i; ++j)
			{
				if (l[j] < t - 1)
				{
					may = false; break;
				}
				else
					q.push_back({ l[j], -j });
			}
			sort(q.begin(), q.end());
			reverse(q.begin(), q.end());
			vector<ll> stack;
			for (int j = t - 1; j <= i; ++j)
			{
				if (j >= t)
				{
					if (!stack.empty() && stack.back() != j)
					{
						may = false;
						break;
					}
					else
					{
						if (!stack.empty())
							stack.pop_back();
					}
				}
				while (!q.empty() && q.back().first == j)
				{
					if (-q.back().second != j)
						stack.push_back(-q.back().second);
					q.pop_back();
				}
			}
			if (may)
			{
				if (i < n - 1)
				{
					p = min(p, { s[i + 1],-i - 1 });
				}
				else
				{
					p = { 'a' - 1,-inf };
				}
			}
			
		}
		if (p.first == 'a'-1)
			break;
		else
		{
			t = -p.second + 1;
			ans.push_back(s[t - 1]);
		}
	}
	cout << ans;

	return 0;
}