#include <algorithm>
#include <cassert>
#include <cstdio>
#include <queue>
#include <string>
#include <tuple>
#include <vector>
#include <map>


using namespace std;

double MAGIC = 1.1;

namespace Brute {
  int k;
  map<pair<vector<int>, bool>, bool> result;
  bool check(vector<int> state, bool move) {
    auto it = result.find({state, move});
    if (it != result.end()) {
      return it->second;
    }
    auto &res = result[{state, move}];
    if (state[0] == 0) {
      return res = move;
    }
    for (int x = 0; x < (int)state.size(); x++) {
      int eaten = 0;
      for (int y = state[x] - 1; y >= 0; y--) {
        for (int zz = x; zz < (int)state.size(); zz++) {
          if (y < state[zz]) eaten++; else break;
        }
        if (eaten <= k || move) {
          vector<int> copy = state;
          for (int zz = x; zz < (int)state.size(); zz++) {
            copy[zz] = min(copy[zz], y);
          }
          if (check(copy, !move) == move) {
            return res = move;
          }
        }
      }
    }
    return res = !move;
  }
}


struct tree {
  vector<pair<int, int>> vals;
  vector<int> p;
  vector<int> sons[2];
  int root = -1;
};

class Game {
  vector<int> cur;

public:
  explicit Game(int n) : cur(n, n) {}

  void do_move(int x, int y) {
    for (int i = x - 1; i < (int) cur.size(); i++) {
      cur[i] = min(cur[i], y - 1);
    }
  }
  int col_height(int x) {
    return cur[x];
  }

  bool check_win_can_be_forced(int i, int j) {
    vector<int> cur_copy = cur;
    do_move(i + 1, j + 1);
    bool res = Brute::check(cur, false);
    cur = cur_copy;
    return res;
  }

  tree build_tree() {
    int n = cur.size();
    vector<vector<int>> rem(n, vector<int>(n));
    for (int i = n - 1; i >= 0; i--) {
      int r = 0;
      for (int j = n - 1; j >= 0; j--) {
        if (j < cur[i]) {
          r++;
          rem[i][j] = ((i == n - 1) ? 0 : rem[i + 1][j]) + r;
        } else {
          rem[i][j] = 0;
        }
      }
    }
    tree res;
    for (int allow_large = 0; allow_large < 2; allow_large++) {
      for (int i = 0; i < n; i++) {
        for (int j = 0; j < n; j++) {
          if (i <= 1 && j <= 1) continue;
          if (rem[i][j] == 0) continue;
          if (rem[i][j] >= 10 || allow_large) continue;
          if (rem[0][0] - rem[i][j] <= 30) {
            if (!check_win_can_be_forced(i, j)) continue;
          }
          res.vals.emplace_back(i + 1, j + 1);
          res.p.emplace_back(-1);
          res.sons[0].emplace_back(-1);
          res.sons[1].emplace_back(-1);
        }
      }
      if (!res.vals.empty()) break;
    }
    std::priority_queue<pair<int, int>, vector<pair<int, int>>, greater<>> q;
    const int MULT = 10000;
    for (int i = 0; i < (int) res.vals.size(); i++) {
      q.emplace(-rem[res.vals[i].first - 1][res.vals[i].second - 1] * MAGIC * MULT, i);
    }
    while (q.size() > 1) {
      auto [val1, v1] = q.top();
      q.pop();
      auto [val2, v2] = q.top();
      q.pop();
      int nid = res.vals.size();
      res.vals.emplace_back(-1, -1);
      res.p[v1] = res.p[v2] = nid;
      res.p.emplace_back(-1);
      res.sons[0].emplace_back(v1);
      res.sons[1].emplace_back(v2);
      q.emplace((val1 + val2) / 2 + MULT, nid);
    }

    if (q.empty()) {
      res.root = -1;
    } else {
      res.root = q.top().second;
    }
    return res;
  }
};

void encode(int n) {
  static char s[1'000'000];
  scanf("%s", s);
  int ptr = 0;


  auto play_one_game = [&](bool move) {
    Game g(n);

    bool skip = false;
    int x, y;
    do {
      if (move) {
        if (skip) {
          int up = g.col_height(0);
          int right = 1;
          while (right < n && g.col_height(right)) right++;
          if (up > right) {
            x = 1;
            y = right + 1;
          } else if (right > up) {
            x = up + 1;
            y = 1;
          } else {
            x = 1;
            y = right;
          }
        } else {
          tree cur_tree = g.build_tree();
          int r = cur_tree.root;
          if (cur_tree.root == -1) {
            x = y = 2;
          } else {
            while (cur_tree.vals[r].first == -1) {
              if (s[ptr] == 0) {
                r = cur_tree.sons[0][r];
              } else {
                r = cur_tree.sons[s[ptr] - '0'][r];
                ptr++;
              }
            }
            std::tie(x, y) = cur_tree.vals[r];
          }
        }
        g.do_move(x, y);
        printf("%d %d\n", x, y);
        fflush(stdout);
      } else {
        assert(scanf("%d%d", &x, &y) == 2);
        g.do_move(x, y);
      }
      if (x <= 2 && y <= 2) {
        skip = true;
      }
      move = !move;
    } while (x != 1 || y != 1);
  };

  bool first = true;
  while (s[ptr]) {
    play_one_game(first);
    first = !first;
    fprintf(stderr, "ptr = %d\n", ptr);
  }
  printf("0\n");
}

void decode(int n, int len) {
  int games;
  scanf("%d", &games);
  string result;
  bool first = true;
  for (int i = 0; i < games; i++) {
    Game g(n);
    int x, y;
    bool skip = false;
    bool move = first;
    first = !first;
    do {
      scanf("%d%d", &x, &y);
      if (x <= 2 && y <= 2) {
        skip = true;
      }
      if (!skip && move) {
        tree t = g.build_tree();
        int id = find(t.vals.begin(), t.vals.end(), pair<int, int>{x, y}) - t.vals.begin();
        assert(id != (int)t.vals.size());
        string app;
        while (t.p[id] != -1) {
          int pid = t.p[id];
          if (t.sons[0][pid] == id) {
            app += '0';
          } else {
            assert(t.sons[1][pid] == id);
            app += '1';
          }
          id = pid;
        }
        reverse(app.begin(), app.end());
        result += app;
      }
      g.do_move(x, y);
      move = !move;
    } while (x != 1 || y != 1);
  }
  assert((int)result.size() >= len);
  result = result.substr(0, len);
  printf("%s\n", result.c_str());
}

int main(int argc, char**argv) {
//  freopen("/tmp/game.out", "w", stdout);
//  freopen("/tmp/game.in", "r", stdin);
  if (argc >= 2) MAGIC = atof(argv[1]);
  int run, n, len;
  scanf("%d%d%d%d", &run, &n, &Brute::k, &len);

  if (run == 1) {
    encode(n);
  } else {
    decode(n, len);
  }
}
