#include <iostream>
#include <vector>
#include <algorithm>
#include <queue>
#include <random>

using namespace std;
using pi = pair<int, int>;

struct Edge {
    int to, cap, flow, nxt;
    Edge(int to, int cap, int flow, int nxt) : to(to), cap(cap), flow(flow), nxt(nxt) { }
    Edge() : to(-1), cap(0), flow(0), nxt(-1) { }
};

struct Network {
    int n, s, t;
    vector<int> head, ptr, dist;
    vector<Edge> edges;

    Network(int n, int s, int t) : n(n), s(s), t(t), head(n,-1), ptr(n,-1), dist(n) { 

    }

    void add_edge(int a, int b, int c) {
        edges.emplace_back(b, c, 0, head[a]); head[a] = edges.size() - 1;
        edges.emplace_back(a, 0, 0, head[b]); head[b] = edges.size() - 1;
    }

    bool bfs() {
        queue<int> q; q.push(s);
        fill(dist.begin(), dist.end(), n+1);
        dist[s] = 0;
        while (!q.empty()) {
            int v = q.front(); q.pop();
            for (int i = head[v]; i != -1; i = edges[i].nxt) {
                if (edges[i].flow == edges[i].cap) continue;
                int to = edges[i].to;
                if (dist[to] > dist[v] + 1) {
                    dist[to] = dist[v] + 1;
                    q.push(to);
                }
            }
        }
        return dist[t] < n;
    }

    int dfs(int v) {
        if (v == t) return 1;
        for (int& i = ptr[v]; i != -1; i = edges[i].nxt) {
            const int to = edges[i].to;
            if (edges[i].flow < edges[i].cap && dist[to] == dist[v] + 1) {
                int push = dfs(to);
                if (push != 0) {
                    edges[i].flow += 1;
                    edges[i^1].flow -= 1;
                    return push;
                }
            }
        }
        return 0;
    }

    int flow() {
        int f = 0;
        while (bfs()) {
            copy(head.begin(), head.end(), ptr.begin());
            int push = 0;
            while (push = dfs(s)) f += push;
        }
        return f;
    }
};

const int N = 3e5+7;

int col[N];
vector<int> g[N];

void paint(int v, int p = -1, int h = 1) {
    col[v] = h;
    for (int to : g[v]) {
        if (to != p) paint(to, v, h ^ 3);
    }
}

int main() {
    int n, m; cin >> n >> m;
    vector<pair<int, int>> edges(m);
    vector<vector<pi>> pin(n), pout(n);
    for (auto& [a, b] : edges) {
        cin >> a >> b; a--, b--;
        g[a].push_back(b);
        g[b].push_back(a);
    }
    for (int i = 0; i < n; ++i) {
        if (!col[i]) paint(i);
    }

    Network net(m+n+2, 0, 1);
    vector<int> p(m); iota(p.begin(), p.end(), 0);
    mt19937 rng(1337228);
    shuffle(p.begin(), p.end(), rng);
    for (int i : p) {
        if (col[edges[i].first] == 1) {
            net.add_edge(0, i+2, 1);
            net.add_edge(i+2, 2+m+edges[i].first, 1); 
            pin[edges[i].first].emplace_back(i, net.edges.size() - 2);
            net.add_edge(i+2, 2+m+edges[i].second, 1);
            pin[edges[i].second].emplace_back(i, net.edges.size() - 2);
        } else {
            net.add_edge(i+2, 1, 1);
            net.add_edge(2+m+edges[i].first, i+2, 1);
            pout[edges[i].first].emplace_back(i, net.edges.size() - 2);
            net.add_edge(2+m+edges[i].second, i+2, 1);
            pout[edges[i].second].emplace_back(i, net.edges.size() - 2);
        }
    }

    int flow = net.flow();
    if (2 * flow == m) {
        cout << "Yes" << endl;
        for (int i = 0; i < n; ++i) {
            vector<int> a, b;
            for (auto [p, q] : pin[i]) if (net.edges[q].flow != 0) a.push_back(p);
            for (auto [p, q] : pout[i]) if (net.edges[q].flow != 0) b.push_back(p);
            for (int j = 0; j < a.size(); ++j) {
                cout << edges[a[j]].first + 1 << " " << edges[a[j]].second + 1 << " "
                    << edges[b[j]].first + 1 << " " << edges[b[j]].second + 1 << endl;
            }
        }
    } else {
        cout << "No" << endl;
    }
}