#include <algorithm>
#include <cassert>
#include <iostream>
#include <numeric>
#include <vector>

using namespace std;

#define SZ(c) (int)c.size()
#define ALL(c) c.begin(), c.end()

// #define int long long

const int N = 3e5 + 7;
const int M = 3e5 + 7;
const int INF = 1e9;

int n, m;
vector<pair<int, int>> g[N];
struct Edge {
  int v, u, w;
} edges[M];

void scan() {
  cin >> n >> m;
  for (int i = 0; i < m; ++i) {
    int v, u, w;
    cin >> v >> u >> w;
    --v, --u;
    g[v].push_back({u, w});
    g[u].push_back({v, w});
    edges[i] = {v, u, w};
  }
}

struct DSU {
  vector<int> par;
  vector<int> sz;

  int find(int x) { return x == par[x] ? x : par[x] = find(par[x]); }

  bool join(int x, int y) {
    x = find(x);
    y = find(y);
    if (x != y) {
      if (sz[x] > sz[y]) {
        swap(x, y);
      }
      sz[y] += sz[x];
      par[x] = y;
    }
    return x != y;
  }

  DSU() {}
  DSU(int n) : par(n), sz(n, 1) { iota(ALL(par), 0); }
};

bool used[M];

void build(int i) {
  fill(used, used + m, false);
  DSU dsu(n);
  for (; i < m; ++i) {
    if (dsu.join(edges[i].v, edges[i].u)) {
      used[i] = true;
    }
  }
}

int dp_min[N];
int dp_max[N];

void dfs(int v, int last, int w) {
  dp_min[v] = (v == last ? INF : min(dp_min[last], w));
  dp_max[v] = (v == last ? 0 : max(dp_max[last], w));
  for (auto e : g[v]) {
    if (e.first != last) {
      dfs(e.first, v, e.second);
    }
  }
}

int ans[N];

signed main() {
  ios_base::sync_with_stdio(false);
  cin.tie(NULL);
  cout.tie(NULL);
  scan();
  fill(ans, ans + n, 2 * INF);
  sort(edges, edges + m, [](Edge a, Edge b) { return a.w < b.w; });
  for (int ei = 0; ei < m; ++ei) {
    build(ei);
    for (int v = 0; v < n; ++v) {
      g[v].clear();
    }
    for (int i = 0; i < m; ++i) {
      if (used[i]) {
        Edge e = edges[i];
        g[e.v].push_back({e.u, e.w});
        g[e.u].push_back({e.v, e.w});
      }
    }
    dfs(0, 0, 0);
    for (int v = 1; v < n; ++v) {
      ans[v] = min(ans[v], dp_min[v] + dp_max[v]);
    }
  }
  for (int v = 1; v < n; ++v) {
    cout << ans[v] << '\n';
  }
  return 0;
}
