#include <algorithm>
#include <cassert>
#include <iostream>
#include <numeric>
#include <vector>

using namespace std;

#define SZ(c) (int)c.size()
#define ALL(c) c.begin(), c.end()

const int N = 3e5 + 7;
const int M = 3e5 + 7;
const int K = 20;
const int INF = 1e9;

int n, m;
vector<pair<int, int>> g[N];
struct Edge {
  int v, u, w;
} edges[M];

void scan() {
  cin >> n >> m;
  for (int i = 0; i < m; ++i) {
    int v, u, w;
    cin >> v >> u >> w;
    --v, --u;
    g[v].push_back({u, w});
    g[u].push_back({v, w});
    edges[i] = {v, u, w};
  }
}

struct DSU {
  vector<int> par;
  vector<int> sz;

  int find(int x) { return x == par[x] ? x : par[x] = find(par[x]); }

  bool join(int x, int y) {
    x = find(x);
    y = find(y);
    if (x != y) {
      if (sz[x] > sz[y]) {
        swap(x, y);
      }
      sz[y] += sz[x];
      par[x] = y;
    }
    return x != y;
  }

  DSU() {}
  DSU(int n) : par(n), sz(n, 1) { iota(ALL(par), 0); }
};

bool used[M];

void build() {
  sort(edges, edges + m, [](Edge a, Edge b) { return a.w > b.w; });
  DSU dsu(n);
  for (int i = 0; i < m; ++i) {
    if (dsu.join(edges[i].v, edges[i].u)) {
      used[i] = true;
    }
  }
}

int par[N][K];
int dp_min[N][K];
int dp_max[N][K];

void dfs(int v, int last, int w) {
  par[v][0] = last;
  dp_min[v][0] = (v == last ? INF : w);
  dp_max[v][0] = (v == last ? 0 : w);
  for (int k = 1; k < K; ++k) {
    par[v][k] = par[par[v][k - 1]][k - 1];
    dp_min[v][k] = min(dp_min[v][k - 1], dp_min[par[v][k - 1]][k - 1]);
    dp_max[v][k] = max(dp_max[v][k - 1], dp_max[par[v][k - 1]][k - 1]);
  }
  for (auto e : g[v]) {
    if (e.first != last) {
      dfs(e.first, v, e.second);
    }
  }
}

signed main() {
  ios_base::sync_with_stdio(false);
  cin.tie(NULL);
  cout.tie(NULL);
  scan();
  build();
  for (int v = 0; v < n; ++v) {
    g[v].clear();
  }
  for (int i = 0; i < m; ++i) {
    if (used[i]) {
      Edge e = edges[i];
      g[e.v].push_back({e.u, e.w});
      g[e.u].push_back({e.v, e.w});
    }
  }
  dfs(0, 0, 0);
  for (int v = 1; v < n; ++v) {
    cout << dp_min[v][K - 1] + dp_max[v][K - 1] << '\n';
  }
  return 0;
}
