#include <iostream>
#include <vector>
#include <algorithm>
#include <set>
#include <map>
#include <ios>
#include <cassert>
using namespace std;

enum class Remaints {
	NOTHING = 0, IN = 1, OUT = 2, FAIL = 3
};

typedef pair<int, int> edge;
vector<pair<edge, edge>> gadgets;

enum class EdgeType {
	BOTH, IN, OUT
};

vector<vector<pair<int, EdgeType>>> g;
set<pair<int, int>> edges;
vector<char> used;

Remaints dfs(int v, int parent = -1) {
	used[v] = true;

	vector<pair<pair<int, EdgeType>, Remaints>> children;
	for (auto [w, direction] : g[v]) {
		if (used[w])
			continue;
		Remaints possible_remaint = dfs(w, v);
		if (possible_remaint == Remaints::FAIL)
			return Remaints::FAIL;
		children.push_back({ {w, direction}, possible_remaint });
	}

	vector<pair<int, EdgeType>> in_children;
	vector<pair<int, EdgeType>> out_children;
	vector<pair<int, EdgeType>> both_children;

	for (auto& [child, possible_remaint] : children) {
		auto [v, edge_type] = child;
		
		if (possible_remaint == Remaints::IN) {
			if (edge_type == EdgeType::IN)
				return Remaints::FAIL;
			else if (edge_type == EdgeType::BOTH)
				in_children.push_back(child);
		}
		else if (possible_remaint == Remaints::OUT) {
			if (edge_type == EdgeType::OUT)
				return Remaints::FAIL;
			else if (edge_type == EdgeType::BOTH)
				out_children.push_back(child);
		}
		else {  // Nothing
			if (edge_type == EdgeType::IN)
				in_children.push_back(child);
			else if (edge_type == EdgeType::OUT)
				out_children.push_back(child);
			else
				both_children.push_back(child);
		}
	}

	Remaints all_possible_remaints[] = { Remaints::NOTHING, Remaints::IN, Remaints::OUT };
	Remaints answer = Remaints::FAIL;

	for (Remaints in_remaints : all_possible_remaints) {
		if (parent == -1 && in_remaints != Remaints::NOTHING)
			continue;
		vector<edge> in_edges;
		vector<edge> out_edges;
		for (auto [w, _] : in_children)
			in_edges.push_back({ w, v });
		for (auto [w, _] : out_children)
			out_edges.push_back({ v, w });
		for (auto [w, _] : both_children) {
			in_edges.push_back({ w, v });
			out_edges.push_back({ v, w });
		}

		if (in_remaints == Remaints::IN)
			in_edges.push_back({ parent, v });
		else if (in_remaints == Remaints::OUT)
			out_edges.push_back({ v, parent });
		
		if (in_edges.size() != out_edges.size())
			continue;

		if (answer != Remaints::FAIL) {
			std::terminate();
		}
		answer = in_remaints;

		for (int i = 0; i < in_edges.size(); i++) {
			gadgets.push_back({ in_edges[i], out_edges[i] });
		}
	}

	return answer;
}

signed main() {
	ios_base::sync_with_stdio(0); cin.tie(0); cout.tie(0);
	int n, m;
	cin >> n >> m;
	g.resize(n);
	used.resize(n);
	

	for (int i = 0; i < m; i++) {
		int a, b;
		cin >> a >> b;
		a--; b--;
		edges.insert({ a, b });
	}

	for (auto [a, b] : edges) {
		bool exists_back = edges.count({ b, a });
		if (!exists_back) {
			g[a].push_back({ b, EdgeType::OUT });
			g[b].push_back({ a, EdgeType::IN });
		}
		else if (a < b) {
			g[a].push_back({ b, EdgeType::BOTH });
			g[b].push_back({ a, EdgeType::BOTH });
		}
	}

	for (int i = 0; i < n; i++) {
		if (used[i])
			continue;
		auto possible_remaints = dfs(i);
		if (possible_remaints != Remaints::NOTHING) {
			cout << "No\n";
			return 0;
		}
	}

	cout << "Yes\n";
	for (auto [e1, e2] : gadgets) {
		if (e1.second != e2.first)
			swap(e1, e2);
		// assert(e1.second == e2.first);
		cout << e1.first + 1 << " " << e1.second + 1 << " " <<
			e2.first + 1 << " " << e2.second + 1 << "\n";
	}
}