#include <iostream>
#include <string>
#include <vector>
#include <algorithm>
#include <map>
#include <numeric>
#include <queue>

using namespace std;

using ll = long long;
template <typename T>
using v = vector<T>;
template <typename A, typename B>
using pa = pair<A, B>;

struct dsu {
	v<ll> parent;
	v<ll> rank;
	v<ll> L;
	v<ll> R;
	dsu(ll n) : parent(n), rank(n, 1), L(n), R(n) {
		iota(parent.begin(), parent.end(), 0ll);
		iota(L.begin(), L.end(), 0ll);
		iota(R.begin(), R.end(), 0ll);
	}

	ll get(ll vert) {
		if (parent[vert] == vert) {
			return vert;
		}
		else {
			return parent[vert] = get(parent[vert]);
		}
	}

	void unite(ll a, ll b) {
		a = get(a);
		b = get(b);
		if (a != b) {
			if (rank[a] < rank[b]) {
				swap(a, b);
			}
			parent[b] = a;
			rank[a] += rank[b];
			L[a] = min(L[a], L[b]);
			R[a] = max(R[a], R[b]);
		}
	}
};

int main() {
	ll n, q;
	cin >> n >> q;
	v<pa<ll, ll>> trees(n);
	for (ll i = 0; i < n; ++i) {
		ll xi, hi;
		cin >> xi >> hi;
		trees[i] = { xi, hi };
	}
	for (ll req = 0; req < q; ++req) {
		ll li, ri;
		cin >> li >> ri;
		--li;
		dsu d(ri - li);
		v<pa<ll, ll>> subarr(trees.begin() + li, trees.begin() + ri);
		v<bool> cut_down(ri - li);
		v<bool> scheduled(ri - li);
		queue<ll> q;
		ll cnt = 0;
		for (ll i = 0; i < subarr.size(); ++i) {
			if (!cut_down[i]) {
				ll L = i - 1;
				if (L >= 0 && cut_down[L]) {
					L = d.L[d.get(L)] - 1;
				}
				bool canLeft = L == -1 || subarr[L].first <= subarr[i].first - subarr[i].second;
				canLeft &= li == 0 || trees[li - 1].first <= subarr[i].first - subarr[i].second;
				canLeft &= trees.front().first <= subarr[i].first - subarr[i].second;
				ll R = i + 1;
				if (R < subarr.size() && cut_down[R]) {
					R = d.R[d.get(R)] + 1;
				}
				bool canRight = R == subarr.size() || subarr[R].first >= subarr[i].first + subarr[i].second;
				canRight &= ri == n || trees[ri].first >= subarr[i].first + subarr[i].second;
				canRight &= trees.back().first >= subarr[i].first + subarr[i].second;
				if (canLeft || canRight) {
					if (!scheduled[i]) {
						scheduled[i] = true;
						q.push(i);
					}
				}
			}
		}
		while (!q.empty()) {
			ll ti = q.front();
			q.pop();
			cut_down[ti] = true;
			if (ti > 0 && cut_down[ti - 1]) {
				d.unite(ti, ti - 1);
			}
			if (ti < subarr.size() - 1 && cut_down[ti + 1]) {
				d.unite(ti, ti + 1);
			}
			{
				ll L = ti - 1;
				if (L >= 0 && cut_down[L]) {
					L = d.L[d.get(L)] - 1;
				}
				ll i = L;
				if (i >= 0) {
					if (!cut_down[i]) {
						ll L = i - 1;
						if (L >= 0 && cut_down[L]) {
							L = d.L[d.get(L)] - 1;
						}
						bool canLeft = L == -1 || subarr[L].first <= subarr[i].first - subarr[i].second;
						canLeft &= li == 0 || trees[li - 1].first <= subarr[i].first - subarr[i].second;
						canLeft &= trees.front().first <= subarr[i].first - subarr[i].second;
						ll R = i + 1;
						if (R < subarr.size() && cut_down[R]) {
							R = d.R[d.get(R)] + 1;
						}
						bool canRight = R == subarr.size() || subarr[R].first >= subarr[i].first + subarr[i].second;
						canRight &= ri == n || trees[ri].first >= subarr[i].first + subarr[i].second;
						canRight &= trees.back().first >= subarr[i].first + subarr[i].second;
						if (canLeft || canRight) {
							if (!scheduled[i]) {
								scheduled[i] = true;
								q.push(i);
							}
						}
					}
				}
			}
			{
				ll R = ti + 1;
				if (R < subarr.size() && cut_down[R]) {
					R = d.R[d.get(R)] + 1;
				}
				ll i = R;
				if (i < subarr.size()) {
					if (!cut_down[i]) {
						ll L = i - 1;
						if (L >= 0 && cut_down[L]) {
							L = d.L[d.get(L)] - 1;
						}
						bool canLeft = L == -1 || subarr[L].first <= subarr[i].first - subarr[i].second;
						canLeft &= li == 0 || trees[li - 1].first <= subarr[i].first - subarr[i].second;
						canLeft &= trees.front().first <= subarr[i].first - subarr[i].second;
						ll R = i + 1;
						if (R < subarr.size() && cut_down[R]) {
							R = d.R[d.get(R)] + 1;
						}
						bool canRight = R == subarr.size() || subarr[R].first >= subarr[i].first + subarr[i].second;
						canRight &= ri == n || trees[ri].first >= subarr[i].first + subarr[i].second;
						canRight &= trees.back().first >= subarr[i].first + subarr[i].second;
						if (canLeft || canRight) {
							if (!scheduled[i]) {
								scheduled[i] = true;
								q.push(i);
							}
						}
					}
				}
			}
			++cnt;
		}
		cout << cnt << '\n';
	}
}